import * as functions from 'firebase-functions';
import { firestore, initializeApp } from 'firebase-admin';

initializeApp(); // setup firebase config

const FIRESTORE_PREFIX = 'waspada/covid19/';

function getNotifikasiId(time : Date, path : String) {
    let newPath = path.startsWith(FIRESTORE_PREFIX) ? path.substr(FIRESTORE_PREFIX.length) : path;
    newPath = newPath.replace(/\//g, '-'); // institusi/47/kebutuhan/9 => institusi-47-kebutuhan-9

    return `${time.toISOString()}-${newPath}`;
}

export const notifyNewInstitusi = functions.firestore
    .document('waspada/covid19/institusi/{id}')
    .onCreate((snapshot, context) => {
        const institusi = snapshot.data();
        const institusiId = context.params.id;

        if (institusi === undefined) {
            console.log('Failed to create new notifikasi for institusi ID:', institusiId);
            return null;
        }

        const notifikasiId = getNotifikasiId((snapshot.createTime ? snapshot.createTime.toDate() : new Date()), snapshot.ref.path);

        return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
            judul: 'Ada Institusi kesehatan baru!',
            deskripsi: `${institusi.nama} (${institusi.provinsi}) baru saja terdaftar. #YukSIGAP cari tahu kebutuhannya!`,
            tipe: 'institusi',
            waktu: snapshot.createTime,
            objectPath: snapshot.ref,
        });
    }
);

export const notifyNewKebutuhan = functions.firestore
    .document('/waspada/covid19/institusi/{institusiId}/kebutuhan/{kebutuhanId}')
    .onCreate((snapshot, context) => {
        const handler = async (_snapshot : firestore.DocumentSnapshot, _context : functions.EventContext) => {
            const kebutuhan = _snapshot.data();
            const institusi = (await _snapshot.ref.parent.parent?.get())?.data();
            const item = (await kebutuhan?.item.get()).data();
            const notifikasiId = getNotifikasiId((snapshot.createTime ? snapshot.createTime.toDate() : new Date()), snapshot.ref.path);

            return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
                judul: `${institusi?.nama} membutuhkan ${kebutuhan?.kebutuhan} ${item.nama}!`,
                deskripsi: `${institusi?.nama} (${institusi?.provinsi}) membutuhkan ${kebutuhan?.kebutuhan} ${item.nama}. #YukSIGAP mari membantu ${institusi?.nama}.`,
                tipe: 'kebutuhan',
                waktu: snapshot.createTime,
                objectPath: snapshot.ref,
            });
        };

        return handler(snapshot, context);
    });

export const notifyNewSupplier = functions.firestore
    .document('waspada/covid19/supplier/{id}')
    .onCreate((snapshot, context) => {
        const supplier = snapshot.data();
        const supplierId = context.params.id;

        if (supplier === undefined) {
            console.log('Failed to create new notifikasi for supplier ID:', supplierId);
            return null;
        }

        const notifikasiId = getNotifikasiId(snapshot.createTime ? snapshot.createTime.toDate() : new Date(), snapshot.ref.path);

        return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
            judul: 'Ada Supplier baru!',
            deskripsi: `${supplier.nama} baru saja terdaftar sebagai supplier di SIGAP. #YukSIGAP cari tahu barang-barang yang disediakan!`,
            tipe: 'supplier',
            waktu: snapshot.createTime,
            objectPath: snapshot.ref,
        });
    }
);

export const notifyNewHomeIndustry = functions.firestore
    .document('waspada/covid19/home-industry/{id}')
    .onCreate((snapshot, context) => {
        const homeIndustry = snapshot.data();
        const homeIndustryId = context.params.id;

        if (homeIndustry === undefined) {
            console.log('Failed to create new notifikasi for homeIndustry ID:', homeIndustryId);
            return null;
        }

        const notifikasiId = getNotifikasiId(snapshot.createTime ? snapshot.createTime.toDate() : new Date(), snapshot.ref.path);

        return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
            judul: 'Ada Home Industry baru!',
            deskripsi: `${homeIndustry.nama} baru saja terdaftar sebagai home industry di SIGAP. #YukSIGAP cari tahu barang-barang yang disediakan!`,
            tipe: 'homeIndustry',
            waktu: snapshot.createTime,
            objectPath: snapshot.ref,
        });
    }
);

export const notifyNewBarangSupplier = functions.firestore
    .document('/waspada/covid19/supplier/{supplierId}/barang/{barangId}')
    .onCreate((snapshot, context) => {
        const handler = async (_snapshot : firestore.DocumentSnapshot, _context : functions.EventContext) => {
            const barang = _snapshot.data();
            const supplier = (await _snapshot.ref.parent.parent?.get())?.data();
            const item = (await barang?.item.get()).data();
            const notifikasiId = getNotifikasiId((snapshot.createTime ? snapshot.createTime.toDate() : new Date()), snapshot.ref.path);

            return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
                judul: `Tersedia ${barang?.stok} ${item.nama}!`,
                deskripsi: `${supplier?.nama} baru saja menambahkan ${barang?.stok} ${item.nama} baru.`,
                tipe: 'supplier-barang',
                waktu: snapshot.createTime,
                objectPath: snapshot.ref,
            });
        };

        return handler(snapshot, context);
    });

export const notifyNewBarangHomeIndustry = functions.firestore
    .document('/waspada/covid19/home-industry/{homeIndustryId}/barang/{barangId}')
    .onCreate((snapshot, context) => {
        const handler = async (_snapshot : firestore.DocumentSnapshot, _context : functions.EventContext) => {
            const barang = _snapshot.data();
            const homeIndsutry = (await _snapshot.ref.parent.parent?.get())?.data();
            const item = (await barang?.item.get()).data();
            const notifikasiId = getNotifikasiId((snapshot.createTime ? snapshot.createTime.toDate() : new Date()), snapshot.ref.path);

            return firestore().doc(FIRESTORE_PREFIX + 'notifikasi/' + notifikasiId).set({
                judul: `Tersedia ${barang?.stok} ${item.nama}!`,
                deskripsi: `${homeIndsutry?.nama} baru saja menambahkan ${barang?.stok} ${item.nama} baru.`,
                tipe: 'homeIndustry-barang',
                waktu: snapshot.createTime,
                objectPath: snapshot.ref,
            });
        };

        return handler(snapshot, context);
    });
